
#import "Friio.h"
#import "io.h"

@implementation Friio

// 設定値定義
// -----------------------------------------------------------
static const int PIPE_REF = 1;
static const int BUFFER_SIZE = 327680;
static const int RESPONSE_SIZE = 16;

static Friios friios[MAX_FRIIOS];

// オープンメソッド
// -----------------------------------------------------------
- (int)open:(int)type
{
	int result = 0;
	int i = 0;

	find_type=type;
	
	getFriios(friios);

	// インターフェースを検索
	result = [self _findDeviceInterface];
	if(!result) {
		return 0;
	}
	for(i=0;i<MAX_FRIIOS;i++){
		if( !friios[i].ERLocationID ) break;
		if( friios[i].FrLocationID==0LL){
			printf("LID=0 status=not+found type=unknown msg=please+reconnect+USB" );
			printf("\n");
		}	
	}	
	return 1;
}




// デバイスリファレンス取得メソッド
// -----------------------------------------------------------
- (int)_findDeviceInterface
{
	SInt32					idVendor = 0x7A69;
    SInt32					idProduct = 0x0001;
	kern_return_t			error;
	CFNumberRef				numberRef;
	CFMutableDictionaryRef 	matchingDictionary = 0;
    io_iterator_t			iterator = 0;
	int						rc = 1;
	
	// マスターポートを生成
	error = IOMasterPort(MACH_PORT_NULL, &masterPort);
	if(error) {
//		printf("_findDevice: Could not open master port.\n");
		return 0;
	}
	
	// 辞書を生成
	matchingDictionary = IOServiceMatching(kIOUSBDeviceClassName);
	
	// ベンダーIDを辞書に登録
	numberRef = CFNumberCreate(kCFAllocatorDefault, kCFNumberSInt32Type, &idVendor);
	CFDictionaryAddValue(matchingDictionary, CFSTR(kUSBVendorID), numberRef);
    CFRelease(numberRef);
	numberRef = 0;
	// プロダクトIDを辞書に登録
    numberRef = CFNumberCreate(kCFAllocatorDefault, kCFNumberSInt32Type, &idProduct);
	CFDictionaryAddValue(matchingDictionary, CFSTR(kUSBProductID), numberRef);
    CFRelease(numberRef);
    numberRef = 0;

	// Friioを探す
	error = IOServiceGetMatchingServices(masterPort, matchingDictionary, &iterator);
	matchingDictionary = 0;
	if(error) {
//		printf("_findDevice: Unable to find the device of the friio.\n");
		return 0;
	}
	
	friio_type=0;
	
	// USBデバイスのリファレンスを取得
	//usbDeviceRef = IOIteratorNext(iterator);
	// インターフェースリファレンスの取得
	while( usbDeviceRef = IOIteratorNext(iterator) ){
		SInt32						score;
		IOReturn					iferror;
		IOCFPlugInInterface			**iodev;
		IOUSBFindInterfaceRequest	interfaceRequest;
		io_iterator_t				ifiterator;
		// プラグインインターフェースの生成
		iferror = IOCreatePlugInInterfaceForService(
			usbDeviceRef,
			kIOUSBDeviceUserClientTypeID,
			kIOCFPlugInInterfaceID,
			&iodev,
			&score
		);
		if(iferror) {
//			printf("_findInterface: Unable to create a device interface. %x\n", error);
			//失敗したら次を探す
			continue;
		}
		iferror = (*iodev)->QueryInterface(
			iodev, 
			CFUUIDGetUUIDBytes(kIOUSBDeviceInterfaceID245),
			(LPVOID)&dev
		);
		IODestroyPlugInInterface(iodev);
		if(iferror || !dev) {
//			printf("_findInterface: Unable to create a device interface.\n");
			//失敗したら次を探す
			continue;
		}
		
//		USBDeviceAddress usbAddress = 0;
		UInt32 locationID = 0;
		
		//(*dev)->GetDeviceAddress (dev, &usbAddress);
		(*dev)->GetLocationID (dev, &locationID);
//		printf ("bus=%d@%08x ",(int) usbAddress, (int) locationID);
		printf ("LID=%08x", (int) locationID);

		// デバイスのオープン
		iferror = (*dev)->USBDeviceOpen(dev);
		if(iferror) {
			(*dev)->Release(dev);
			printf(" status=AccessError");
			printf("\n");
			//失敗したら次を探す
			continue;
		}
		
		// インターフェースイテレータの生成
		interfaceRequest.bInterfaceClass = kIOUSBFindInterfaceDontCare;
		interfaceRequest.bInterfaceSubClass = kIOUSBFindInterfaceDontCare;
		interfaceRequest.bInterfaceProtocol = kIOUSBFindInterfaceDontCare;
		interfaceRequest.bAlternateSetting = kIOUSBFindInterfaceDontCare;
		error = (*dev)->CreateInterfaceIterator(dev, &interfaceRequest, &ifiterator);
//		(*dev)->ResetDevice (dev);
		(*dev)->USBDeviceClose(dev);
		(*dev)->Release(dev);

		if(error) {
//			printf("_findInterface: Unable to create interface iterator.\n");
//			(*dev)->USBDeviceClose(dev);
//			(*dev)->Release(dev);
			//失敗したら次を探す
			printf(" status=AccessError");
			printf("\n");
			continue;
		}
		
		usbInterfaceRef = IOIteratorNext(ifiterator);
		
		// 後処理
		IOObjectRelease(ifiterator);
		ifiterator = 0;

		// インターフェースをセットアップ
		int result = [self _setupInterface];
		if(!result) {
			int i=0;
			while(friios[i].ERLocationID){
				if( friios[i].FrLocationID==locationID ){
					switch(friios[i].cardtype){
					case 0:
						printf(" type=unknown");
						break;
					case 1:
						printf(" type=white");
						break;
					case 2:
						printf(" type=black");
						break;
					}
					break;
				}
				i++;
				if(i==MAX_FRIIOS) break;
			}
			//失敗したら次を探す
			IOObjectRelease(usbDeviceRef);
			printf("\n");
			continue;
		}

		//初期化A
		[self initStageA];
		
		switch( friio_type ){
		case FRIIO_WHITE:
			printf(" type=white");
			break;
		case FRIIO_BLACK:
			printf(" type=black");
		}
		// ストリームの初期化
		[self initStream];

		// 初期化B
		[self initStageB];
		
		// 閉じる
		[self close];

		printf("\n");
		IOObjectRelease(usbDeviceRef);
		continue;
		
	}
	if(!usbDeviceRef) rc=0;
		
	// 後処理
	IOObjectRelease(iterator);
	iterator = 0;
	
    mach_port_deallocate(mach_task_self(), masterPort);
	return 1;
}


// インターフェースセットアップメソッド
// -----------------------------------------------------------
- (int)_setupInterface
{
    SInt32					score;
	IOReturn				error;
    IOCFPlugInInterface 	**iodev;

	// プラグインインターフェースの生成
	error = IOCreatePlugInInterfaceForService(
		usbInterfaceRef,
		kIOUSBInterfaceUserClientTypeID,
		kIOCFPlugInInterfaceID,
		&iodev,
		&score
	);
	IOObjectRelease(usbInterfaceRef);
	usbInterfaceRef=0;
	error = (*iodev)->QueryInterface(
		iodev,
		CFUUIDGetUUIDBytes(kIOUSBInterfaceInterfaceID245),
		(LPVOID)&interface
	);
	if(iodev) (*iodev)->Release(iodev);
	//IODestroyPlugInInterface(iodev);
	if(error || !interface){
//		printf("_setupInterface: Unable to create a device interface.\n");
		printf(" status=AccessError");
		return 0;
    }
	
	// インターフェースを開く
	error = (*interface)->USBInterfaceOpen(interface);
	if(error) {
		(*interface)->Release(interface);
		interface=0;
		if (error == kIOReturnExclusiveAccess) {
			printf(" status=inuse");
			//printf("_setupInterface: interface already in use. %x\n", error);
			//使用中のデバイスはそのまま放置。
		} else {
			printf(" status=AccessError");
			//printf("_setupInterface: Unable to open interface. %x\n", error);
		}
		//printf("_setupInterface: Unable to open interface.\n");
		return 0;
    }
	
	printf(" status=standby");
	
	return 1;
}


// クローズメソッド
// -----------------------------------------------------------
- (int)close
{
    IOReturn error;
	
	if(interface){
		// インターフェースを閉じる
		error = (*interface)->USBInterfaceClose(interface);
		if(error) {
			printf("close: Unable to close interface.\n");
			return 0;
		}
		// インターフェースを解放
		error = (*interface)->Release(interface);
		if(error) {
			printf("close: Unable to release interface.\n");
			return 0;
		}
	}

	return 1;
}


// リクエスト送信関数
// -----------------------------------------------------------
- (int)_sendRequest:(UInt16)wValue withIndex:(UInt16)wIndex
{
	unsigned char	iobuff[32];
	IOReturn		error;
	
	// リクエストを生成
	IOUSBDevRequest req;
	req.bmRequestType = USBmakebmRequestType(kUSBOut, kUSBVendor, kUSBDevice);
	req.bRequest = 0x01;
	req.wValue = wValue;
	req.wIndex = wIndex;
	req.wLength = 0;
	req.wLenDone = 0;
	req.pData = iobuff;
	
	// Friioに送信
	error = (*interface)->ControlRequest(interface, 0, &req);
	if(error) {
		return 0;
	}
	
	return 1;
}	


// 初期化処理Aメソッド
// -----------------------------------------------------------
- (int)initStageA
{
	int			error_flag;
	IOReturn	error;

	// リクエスト送信
	error_flag = [self _sendRequest:0x0002 withIndex:0x0011];
	// リクエストの送信に失敗したら初期化失敗
	if(!error_flag) {
		return 0;
	}
	[self _sendRequest:0x0000 withIndex:0x0011];
	
	unsigned char iobuff1[32];
	IOUSBDevRequest req1;
	
	memset(iobuff1, 0, sizeof(iobuff1));
	iobuff1[0]=0xC1;
	req1.bmRequestType = USBmakebmRequestType(kUSBOut, kUSBVendor, kUSBDevice);
	req1.bRequest = 0x03;
	req1.wValue = 0x3000;
	req1.wIndex = 0x00FE;
	req1.wLength = 1;
	req1.wLenDone = 0;
	req1.pData = iobuff1;
	error = (*interface)->ControlRequest(interface, 0, &req1);

	iobuff1[0]=0;
	req1.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBVendor, kUSBDevice);
	req1.bRequest = 0x02;
	req1.wValue = 0x3000;
	req1.wIndex = 0x0100;
	req1.wLength = 1;
	req1.wLenDone = 0;
	req1.pData = iobuff1;
	error = (*interface)->ControlRequest(interface, 0, &req1);

	//White Friio
	if(iobuff1[0]){
		friio_type=FRIIO_WHITE;
		//fprintf(stderr,"WHITE\n");
	}else{	
		friio_type=FRIIO_BLACK;
		//fprintf(stderr,"BLACK\n");
		unsigned char iobuff2[32];
		IOUSBDevRequest req2;
		req2.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBVendor, kUSBDevice);
		req2.bRequest = 0x02;
		req2.wValue = 0x3000;
		req2.wIndex = 0x0080;
		req2.wLength = 1;
		req2.wLenDone = 0;
		req2.pData = iobuff2;
		error = (*interface)->ControlRequest(interface, 0, &req2);
	}
	
	[self _sendRequest:0x0004 withIndex:0x0030];
	
	return 1;
}


// 初期化処理Aメソッド
// -----------------------------------------------------------
- (int)initStageB
{
	if(friio_type==FRIIO_WHITE){
		//White Friio initstageB
		[self _sendRequest:0x3040 withIndex:0x01];
		[self _sendRequest:0x3038 withIndex:0x04];
		[self _sendRequest:0x3040 withIndex:0x05];
		[self _sendRequest:0x3040 withIndex:0x07];
		[self _sendRequest:0x304F withIndex:0x0F];
		[self _sendRequest:0x3021 withIndex:0x11];
		[self _sendRequest:0x300B withIndex:0x12];
		[self _sendRequest:0x302F withIndex:0x13];
		[self _sendRequest:0x3031 withIndex:0x14];
		[self _sendRequest:0x3002 withIndex:0x16];
		[self _sendRequest:0x30C4 withIndex:0x21];
		[self _sendRequest:0x3020 withIndex:0x22];
		[self _sendRequest:0x3079 withIndex:0x2C];
		[self _sendRequest:0x3034 withIndex:0x2D];
		[self _sendRequest:0x3000 withIndex:0x2F];
		[self _sendRequest:0x3028 withIndex:0x30];
		[self _sendRequest:0x3031 withIndex:0x31];
		[self _sendRequest:0x30DF withIndex:0x32];
		[self _sendRequest:0x3001 withIndex:0x38];
		[self _sendRequest:0x3078 withIndex:0x39];
		[self _sendRequest:0x3033 withIndex:0x3B];
		[self _sendRequest:0x3033 withIndex:0x3C];
		[self _sendRequest:0x3090 withIndex:0x48];
		[self _sendRequest:0x3068 withIndex:0x51];
		[self _sendRequest:0x3038 withIndex:0x5E];
		[self _sendRequest:0x3000 withIndex:0x71];
		[self _sendRequest:0x3008 withIndex:0x72];
		[self _sendRequest:0x3000 withIndex:0x77];
		[self _sendRequest:0x3021 withIndex:0xC0];
		[self _sendRequest:0x3010 withIndex:0xC1];
		[self _sendRequest:0x301A withIndex:0xE4];
		[self _sendRequest:0x301F withIndex:0xEA];
		[self _sendRequest:0x3000 withIndex:0x77];
		[self _sendRequest:0x3000 withIndex:0x71];
		[self _sendRequest:0x3000 withIndex:0x71];
		[self _sendRequest:0x300C withIndex:0x76];
	}else{
		//Black friio initstageB
		[self _sendRequest:0x3040 withIndex:0x0001];
		[self _sendRequest:0x3030 withIndex:0x0004];
		[self _sendRequest:0x3040 withIndex:0x0005];
		[self _sendRequest:0x3040 withIndex:0x0007];
		[self _sendRequest:0x30ff withIndex:0x000f];
		[self _sendRequest:0x3027 withIndex:0x0011];
		[self _sendRequest:0x302f withIndex:0x0013];
		[self _sendRequest:0x3031 withIndex:0x0014];
		[self _sendRequest:0x3002 withIndex:0x0016];
		[self _sendRequest:0x3020 withIndex:0x0022];
		[self _sendRequest:0x3070 withIndex:0x0024];
		[self _sendRequest:0x3079 withIndex:0x002c];
		[self _sendRequest:0x3034 withIndex:0x002d];
		[self _sendRequest:0x3028 withIndex:0x0030];
		[self _sendRequest:0x3031 withIndex:0x0031];
		[self _sendRequest:0x3001 withIndex:0x0038];
		[self _sendRequest:0x3078 withIndex:0x0039];
		[self _sendRequest:0x3090 withIndex:0x0048];
		[self _sendRequest:0x3068 withIndex:0x0051];
		[self _sendRequest:0x3038 withIndex:0x005e];
		[self _sendRequest:0x3021 withIndex:0x00c0];
		[self _sendRequest:0x3010 withIndex:0x00c1];
		[self _sendRequest:0x301a withIndex:0x00e4];
		[self _sendRequest:0x301f withIndex:0x00ea];	
	}
	return 1;
}


// ストリーム初期化メソッド
// -----------------------------------------------------------
- (int)initStream
{
	[self _sendRequest:0x08 withIndex:0x33];
	[self _sendRequest:0x40 withIndex:0x37];
	[self _sendRequest:0x1F withIndex:0x3A];
	[self _sendRequest:0xFF withIndex:0x3B];
	[self _sendRequest:0x1F withIndex:0x3C];
	[self _sendRequest:0xFF withIndex:0x3D];
	[self _sendRequest:0x00 withIndex:0x38];
	[self _sendRequest:0x00 withIndex:0x35];
	[self _sendRequest:0x00 withIndex:0x39];
	[self _sendRequest:0x00 withIndex:0x36];
	return 1;
}



@end
