// cc io.c -O2 -framework CoreFoundation -framework IOKit


/* include CoreFoundation.etc header */

#include <CoreFoundation/CoreFoundation.h>
#include <IOKit/IOCFSerialize.h>
#include <IOKit/IOKitLib.h>
enum {
  kIOServiceInactiveState     = 0x00000001,
  kIOServiceRegisteredState   = 0x00000002,
  kIOServiceMatchedState      = 0x00000004,
  kIOServiceFirstPublishState = 0x00000008,
  kIOServiceFirstMatchState   = 0x00000010
};
#include <sys/ioctl.h>                                // (TIOCGWINSZ, ...)
#include <term.h>                                     // (tputs, ...)
#include <unistd.h>                                   // (getopt, ...)

#include "io.h"
/* */


static int depth;

static UInt64    CurProductID;
static UInt64    CurVendorID;
static UInt64    CurLocationID;

static Friios *friios;

static UInt32	friio_cnt=0;

static UInt32 cur_level=0;

struct context
{
    io_registry_entry_t service;
    UInt32              serviceDepth;
};

static void assertion(int condition, char * message)
{
    if (condition == 0)
    {
        fprintf(stderr, "io: error: %s.\n", message);
        exit(1);
    }
}

static UInt64 getNumber(CFTypeRef object){
    long long number;
    CFTypeID type = CFGetTypeID(object);
    if( type == CFNumberGetTypeID() ){
        CFNumberGetValue(object, kCFNumberLongLongType, &number);
        return (UInt64)number;
    }
    return 0;
}

static void getitem(const void * key, const void * value, void * parameter)
{
    const char * c;

    CFTypeID type = CFGetTypeID(key);
    if( type == CFStringGetTypeID() ){
        c = CFStringGetCStringPtr(key, kCFStringEncodingMacRoman);
        if( strcmp( c, "idVendor" ) == 0) CurVendorID=getNumber(value);
        if( strcmp( c, "idProduct" ) == 0) CurProductID=getNumber(value);
        if( strcmp( c, "locationID" ) == 0) CurLocationID=getNumber(value);
    }
}

static void show( io_registry_entry_t service,
                  UInt32              serviceDepth
                  )
{
    io_name_t              name;           // (don't release)
    struct context         context    = { service, serviceDepth };
    CFMutableDictionaryRef properties = 0;
    kern_return_t          status     = KERN_SUCCESS;

    //friios counting
    if( cur_level != serviceDepth ){
        if( friios[friio_cnt].ERLocationID ){
            friio_cnt++;
        }
        cur_level = serviceDepth;
    }

    status = IORegistryEntryGetNameInPlane(service, "IOUSB", name);
    assertion(status == KERN_SUCCESS, "can't obtain name");
//    printf("%s", name);

//    status = IORegistryEntryGetLocationInPlane(service, "IOUSB", location);
//    if (status == KERN_SUCCESS)  printf("@%s", location);

if(1)
    {
        status = IORegistryEntryCreateCFProperties( service,
                                                    &properties,
                                                    kCFAllocatorDefault,
                                                    kNilOptions );
        assertion(status == KERN_SUCCESS, "can't obtain properties");
        assertion(CFGetTypeID(properties) == CFDictionaryGetTypeID(), NULL);

        CFDictionaryApplyFunction(properties, getitem, &context);

        CFRelease(properties);
    }

//	printf("%qx %qx %qx ", CurVendorID, CurLocationID, CurProductID );

	if( CurVendorID == 31337LL ) // 7a69/1 friio found.
	{
		friios[friio_cnt].FrLocationID = CurLocationID;
	}
	if( CurVendorID == 1423LL ) // 58f Hub or EMVsmartcardreader found
	{
		if( CurProductID == 38176LL ) //9520(黒) black type cardreader
		{
			friios[friio_cnt].cardtype=2;
			friios[friio_cnt].ERLocationID = CurLocationID;
		}
		if( CurProductID == 38178LL ) //9528(白) white type cardreader
		{
			friios[friio_cnt].cardtype=1;
			friios[friio_cnt].ERLocationID = CurLocationID;
		}
	}
}

static void scan( io_registry_entry_t service,
                  Boolean             serviceHasMoreSiblings,
                  UInt32              serviceDepth,
				  int				  depth
                  )
{
    io_registry_entry_t child       = 0;
    io_registry_entry_t childUpNext = 0;
    io_iterator_t       children    = 0;
    kern_return_t       status      = KERN_SUCCESS;


    status = IORegistryEntryGetChildIterator(service, "IOUSB", &children);
    assertion(status == KERN_SUCCESS, "can't obtain children");

    childUpNext = IOIteratorNext(children);

    show(service, serviceDepth );

    if (depth > serviceDepth + 1)
    {
        while (childUpNext)
        {
            child       = childUpNext;
            childUpNext = IOIteratorNext(children);

            scan( /* service                */ child,
                  /* serviceHasMoreSiblings */ (childUpNext) ? TRUE : FALSE,
                  /* serviceDepth           */ serviceDepth + 1,
                  /* depth            */  depth );

            IOObjectRelease(child); child = 0;
        }
    }

    IOObjectRelease(children); children = 0;
}


int getFriios(Friios* pfriios){
//    int i;
//	char *ftype[]={"any","white","black"};
    io_registry_entry_t service  = 0; // (needs release)
    depth=0;
	cur_level=0;
	friios=pfriios;
	memset(friios,0,MAX_FRIIOS*sizeof(Friios));
    service = IORegistryGetRootEntry(kIOMasterPortDefault);
    assertion(service, "can't obtain I/O Kit's root service");
    scan(service,FALSE,0,UINT32_MAX);
    IOObjectRelease(service); service = 0;
	//printf("%d", friio_cnt );
	return 0;
/*
	if(!ULID){
		for(i=0;i<friio_cnt;i++){
			printf("LID=%x status=unknown type=%s", friios[i].FrLocationID, ftype[friios[i].cardtype]  );
			printf("\n");
		}
	}else{
		for(i=0;i<friio_cnt;i++){
			if( ULID==friios[i].FrLocationID ){
				printf("%s", ftype[friios[i].cardtype]  );
			}
		}
	}
	return 0;
*/
}
