#import <Foundation/Foundation.h>
#import "Friio.h"

int main (int argc, const char * argv[]) {
	[NSApplication sharedApplication];
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	int	i, error_flag = 1;
	float signal = 0;
	
	// コマンド引数のチェック
	if(argc < 3) {
		printf("Usage: %s recsec channel outputfile [b25] [friiotype [LocationID]]\n", argv[0]);
		printf("BS/CS Channels(black type friio only):\n");
		printf("  13 : BS-Asahi      24 : 110CS #1\n");
		printf("  14 : BS-i          25 : 110CS #2\n");
		printf("  15 : WOWOW         26 : 110CS #3\n");
		printf("  16 : BS-Japan      27 : 110CS #4\n");
		printf("  17 : BS11          28 : 110CS #5\n");
		printf("  18 : Star Channel  29 : 110CS #6\n");
		printf("  19 : TwellV        30 : 110CS #7\n");
		printf("  20 : BS-NTV        31 : 110CS #8\n");
		printf("  21 : BS-Fuji       32 : 110CS #9\n");
		printf("  22 : NHK BS1/BS2   33 : 110CS #a\n");
		printf("  23 : NHK hi        34 : 110CS #b\n");
		printf("                     35 : 110CS #c\n");
		printf("b25: \n");
		printf("  b25 : use b25     nob25 : no use b25 \n");
		printf("friiotype: \n");
		printf("  0 : Any   1 : White(digital)  2 : Black(BS/CS)\n");
		exit(1);
	}

	// 引数を受け取り
	int recsec  = atoi(argv[1]);
	int channel = atoi(argv[2]);
	const char *output_path = argv[3];
	const char *lid=0;
	int type=FRIIO_ANY;
	i=4;
	int	useb25=1;;

	if( argc>i ) {
		if(!strcmp(argv[i],"nob25")){
			useb25 = 0;
			i++;
		}else if(!strcmp(argv[i],"b25")){
			useb25 = 1;
			i++;
		}
	}
	if( argc>i ) {
		type = atoi(argv[i]);
	}
	i++;
	if( argc>i ) {
		lid = argv[i];
	}
	
	// 引数の値チェック
	if(type==FRIIO_BLACK){
		if(channel < 13 || 35 < channel) {
			printf("Channel number was wrong.\n");
			exit(1);	
		}
	}else{
		if(channel < 13 || 62 < channel) {
			printf("Channel number was wrong.\n");
			exit(1);	
		}
	}
	if(recsec < 0) {
		printf("Recsec was wrong.\n");
		exit(1);	
	}
	
	// Friioインスタンスを生成
	Friio* friio = [Friio alloc];
	[friio setb25:useb25];
	
	// Friioを開く
	error_flag = [friio open:type lid:lid];
	if(!error_flag) {
		printf("No friios available.\n");
		exit(1);
	}
	// 初期化完了
	printf("Initialization complete.\n");

	// LEDを変更
	[friio setLED:1]; // 緑
	
	// チャンネルを変更
	error_flag = [friio setChannel:channel];
	// チャンネル変更で失敗したらエラー
	if(!error_flag) {
		[friio setLED:0];
		exit(1);
	}
	printf("Set channel: %d\n", channel);
	sleep(1);
	
	for(i=1; i<=3; i++) {
		// シグナルを取得
		signal = [friio getSignalLevel];
		printf("Signal level: %f\n", signal);
		// シグナル値のチェック
		if(signal > 10) {
			break;
		}
		sleep(1);
	}
	if(signal < 10) {
		printf("Signal level is insufficient.\n");
		[friio setLED:0];
		exit(1);
	}

	// LEDを変更
	[friio setLED:2]; // 赤

	// 録画実行
//	printf("Recording");
	fflush(stdout);
	[friio rec:output_path withTime:recsec];
	printf("done.\n");
	
	// LEDを変更
	[friio setLED:0]; // 紫

	// Friioを閉じる
	error_flag = [friio close];
	if(!error_flag) {
		printf("Unable to close the friio.\n");
	}
	[friio release];
	
    [pool drain];
	
    return 0;
}
